@@ -172,6 +172,51 @@ static void _midgard_connection_dispose(GObject *object)
 	}
 }
 
+/* Properties */
+enum {
+	PROPERTY_CONFIG = 1
+};
+
+static void
+_midgard_connection_set_property (GObject *object, guint property_id,
+		const GValue *value, GParamSpec *pspec) {
+
+	MidgardConnection *self = (MidgardConnection *) object;
+
+	switch (property_id) {
+
+		case PROPERTY_CONFIG:
+			/* Read only */
+			break;
+
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object,property_id,pspec);
+			break;
+	}
+}
+
+static void
+_midgard_connection_get_property (GObject *object, guint property_id,
+		GValue *value, GParamSpec *pspec) {
+
+	MidgardConnection *self = (MidgardConnection *) object;
+
+	switch (property_id) {
+
+		case PROPERTY_CONFIG:
+			if (!self->priv->config)
+				return;
+			MidgardConfig *config = midgard_config_copy (self->priv->config);
+			g_object_set (config, "dbuser", "", "dbpass", "", NULL);
+			g_value_set_object (value, config);
+			break;
+
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object,property_id,pspec);
+			break;
+	}
+}
+
 static void _midgard_connection_class_init(
 		gpointer g_class, gpointer g_class_data)
 {
@@ -180,6 +225,21 @@ static void _midgard_connection_class_init(
 
 	gobject_class->finalize = _midgard_connection_finalize;
 	gobject_class->dispose = _midgard_connection_dispose;
+     	gobject_class->set_property = _midgard_connection_set_property;
+	gobject_class->get_property = _midgard_connection_get_property;
+
+	/* properties */
+	GParamSpec *pspec = g_param_spec_object ("config",
+			"Config associated with MidgardConnection",
+			"A deep copy of config with empty database username and password values.",
+			MIDGARD_TYPE_CONFIG,
+			G_PARAM_READABLE);
+	/**
+	 * MidgardConnection:config:
+	 * A deep copy of #MidgardConfig associated with #MidgardConnection.
+	 * Returned config property has empty database username and password values set. 
+	 */
+	g_object_class_install_property (gobject_class, PROPERTY_CONFIG, pspec);
 
 	/* signals */
 	/**
@@ -348,6 +408,26 @@ __mysql_reconnect (MidgardConnection *mgd)
 	return FALSE;
 }
 
+
+/* Reopen connection if there's already gda connection asigned 
+ * and midgard connection is not connected. */
+#define __SELF_REOPEN(__self, __retval) { \
+	if (!__self->priv->connected) {\
+		if (__self->priv->connection \
+				&& GDA_IS_CONNECTION (__self->priv->connection)) { \
+			GError *__error = NULL; \
+			__retval = gda_connection_open (__self->priv->connection, &__error); \
+			if (__retval) \
+				g_signal_emit (__self, MIDGARD_CONNECTION_GET_CLASS (__self)->signal_id_connected, 0); \
+			else \
+				MIDGARD_ERRNO_SET_STRING (__self, MGD_ERR_NOT_CONNECTED, \
+						__error && __error->message ? __error->message : "Unknown reason"); \
+			if (__error) \
+				g_clear_error (&__error); \
+		}\
+	}\
+}
+
 /**
  * Adds a named parameter to the given libgda connection string.
  
@@ -600,6 +680,11 @@ midgard_connection_open (MidgardConnection *self, const char *name, GError **err
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
 	MIDGARD_ERRNO_SET (self, MGD_ERR_OK);
+	gboolean rv = TRUE;
+	
+	__SELF_REOPEN (self, rv);
+	if (!rv)
+		return rv;	
 
 	/* FIXME, it should be handled by GError */
 	if(self->priv->config != NULL){
@@ -629,7 +714,6 @@ midgard_connection_open (MidgardConnection *self, const char *name, GError **err
 	self->priv->config = config;
 
 	GHashTable *hash = NULL;
-	gboolean rv = TRUE;
 
 	if(!__midgard_connection_open(self, &hash, TRUE)) {
 
@@ -661,6 +745,12 @@ gboolean midgard_connection_open_from_file(
 	g_assert(mgd != NULL);
 	g_assert (filepath != NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+	gboolean rv = TRUE;
+
+	__SELF_REOPEN (mgd, rv);
+	if (!rv) 
+		return rv;
 	
 	/* FIXME, it should be handled by GError */
 	if(mgd->priv->config != NULL){
@@ -689,7 +779,6 @@ gboolean midgard_connection_open_from_file(
 	mgd->priv->config = config;
 
 	GHashTable *hash = NULL;
-	gboolean rv = TRUE;
 
 	if(!__midgard_connection_open(mgd, &hash, TRUE)) 
 		rv = FALSE;
@@ -785,6 +874,11 @@ gboolean midgard_connection_open_config(
 	g_assert(config != NULL);
 
 	MidgardConfig *self_config = self->priv->config;
+	gboolean rv = TRUE;
+
+	__SELF_REOPEN (self, rv);
+	if (!rv) 
+		return rv;
 
 	/* Emulate the same config pointer, as we have copy associated */
 	if (self_config 
@@ -800,7 +894,6 @@ gboolean midgard_connection_open_config(
 	self->priv->config = midgard_config_copy (config);
 
 	GHashTable *hash = NULL;
-	gboolean rv = TRUE;
 
 	if(!__midgard_connection_open(self, &hash, TRUE))
 		rv = FALSE;
@@ -825,6 +918,31 @@ gboolean midgard_connection_struct_open_config(
 	return TRUE;
 }
 
+/**
+ * midgard_connection_close:
+ * @self: #MidgardConnection instance
+ *
+ * Closes connection to underlying storage. 
+ * All private and public data remains unchanged, so connection might be reopened at any time.
+ * After closing connection, 'disconnected' signal is emitted.
+ *
+ * Since: 10.05.1
+ */
+void
+midgard_connection_close (MidgardConnection *self)
+{
+	g_return_if_fail (self != NULL);
+
+	if (!self->priv->connection ||
+			(!GDA_IS_CONNECTION (self->priv->connection)))
+		return;
+
+ 	gda_connection_close_no_warning (self->priv->connection);
+	g_signal_emit (self, MIDGARD_CONNECTION_GET_CLASS (self)->signal_id_disconnected, 0);
+
+	return;
+}
+
 /**
  * midgard_connection_set_loglevel:
  * @self: #MidgardConnection instance
