@@ -65,6 +65,7 @@ struct _MidgardQueryStoragePrivate {
 	MidgardDBObjectClass *klass;
 	gchar *table_alias;
 	const gchar *table;
+	const gchar *classname;
 };
 
 struct _MidgardQueryPropertyPrivate {
@@ -76,6 +77,7 @@ struct _MidgardQueryPropertyPrivate {
 struct _MidgardQueryExecutorPrivate {
 	MidgardConnection *mgd;
 	MidgardQueryStorage *storage;
+	guint n_constraints;
 	MidgardQueryConstraintSimple *constraint;
 	GSList *orders;
 	GSList *joins;
@@ -88,6 +90,7 @@ struct _MidgardQueryExecutorPrivate {
 	GdaSqlStatement *stmt;
 	guint results_count;
 	gboolean read_only;
+	gboolean include_deleted;
 };
 
 struct _MidgardQueryConstraintSimplePrivate {
@@ -95,9 +98,14 @@ struct _MidgardQueryConstraintSimplePrivate {
 			GdaSqlStatement *stm, GdaSqlExpr *where_expr_node);
 };
 
-#define MQE_SET_TABLE_ALIAS(executor, table) \
-	if (!table) \
-		table = g_strdup_printf ("t%d", ++executor->priv->tableid);
+#define MQE_SET_TABLE_ALIAS(__executor, __storage) \
+	if (__executor->priv->storage == __storage) {\
+		g_free (__storage->priv->table_alias) ;\
+		__storage->priv->table_alias = g_strdup_printf ("t%d", executor->priv->tableid); \
+	} else { \
+		if (!__storage->priv->table_alias) \
+			__storage->priv->table_alias = g_strdup_printf ("t%d", ++executor->priv->tableid); \
+	}
 
 MidgardDBJoin	*midgard_core_dbjoin_new	(void);
 void		midgard_core_dbjoin_free	(MidgardDBJoin *mdbj);
