@@ -39,9 +39,8 @@ midgard_query_select_new (MidgardConnection *mgd, MidgardQueryStorage *storage)
 	g_return_val_if_fail (mgd != NULL, NULL);
 	g_return_val_if_fail (storage != NULL, NULL);
 
-	MidgardQuerySelect *self = g_object_new (MIDGARD_TYPE_QUERY_SELECT, NULL);
-	MIDGARD_QUERY_EXECUTOR (self)->priv->mgd = mgd;
-	MIDGARD_QUERY_EXECUTOR (self)->priv->storage = storage;
+	MidgardQuerySelect *self = g_object_new (MIDGARD_TYPE_QUERY_SELECT, 
+		"connection", mgd, "storage", storage, NULL);
 
 	return self;
 }
@@ -51,8 +50,11 @@ _midgard_query_select_set_constraint (MidgardQueryExecutor *self, MidgardQueryCo
 {
 	g_return_val_if_fail (self != NULL, FALSE);
 	g_return_val_if_fail (constraint != NULL, FALSE);
-	
-	self->priv->constraint = constraint;
+
+	if (self->priv->constraint)
+		g_object_unref (self->priv->constraint);	
+
+	self->priv->constraint = g_object_ref (constraint);
 
 	return TRUE;
 }
@@ -271,7 +273,7 @@ gboolean __query_select_add_joins (MidgardQuerySelect *self)
 		join->position = ++executor->priv->joinid;
 
 		/* Add right storage to targets */
-		MQE_SET_TABLE_ALIAS (executor, right_storage->priv->table_alias);
+		MQE_SET_TABLE_ALIAS (executor, right_storage);
 		gda_sql_select_from_take_new_join (from , join);
 		GdaSqlSelectTarget *s_target = gda_sql_select_target_new (GDA_SQL_ANY_PART (from));
 		s_target->table_name = g_strdup (right_storage->priv->table);
@@ -335,6 +337,17 @@ __add_dummy_constraint (GdaSqlStatementSelect *select, GdaSqlOperation *top_oper
 	return;
 }
 
+static void
+__add_second_dummy_constraint (GdaSqlStatementSelect *select, GdaSqlOperation *top_operation)
+{
+	GdaSqlExpr *dexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (top_operation));
+	dexpr->value = gda_value_new (G_TYPE_STRING);
+	g_value_take_string (dexpr->value, g_strdup ("0<1"));
+	top_operation->operands = g_slist_append (top_operation->operands, dexpr);
+
+	return;
+}
+
 gboolean 
 _midgard_query_select_execute (MidgardQueryExecutor *self)
 {
@@ -401,18 +414,23 @@ _midgard_query_select_execute (MidgardQueryExecutor *self)
 	GdaSqlOperation *operation = where->cond;
 
 	/* Add constraints' conditions (WHERE a=1, b=2...) */
-	if (MIDGARD_QUERY_EXECUTOR (self)->priv->constraint)
-		MIDGARD_QUERY_CONSTRAINT_SIMPLE_GET_INTERFACE (MIDGARD_QUERY_EXECUTOR (self)->priv->constraint)->priv->add_conditions_to_statement (
-				MIDGARD_QUERY_EXECUTOR (self), MIDGARD_QUERY_EXECUTOR (self)->priv->constraint, sql_stm, base_where);
-	else 
-		__add_dummy_constraint (sss, operation); /* no constraints, add dummy WHERE 1=1 */	
+	if (MIDGARD_QUERY_EXECUTOR (self)->priv->constraint) {
+		MIDGARD_QUERY_CONSTRAINT_SIMPLE_GET_INTERFACE (MIDGARD_QUERY_EXECUTOR (self)->priv->constraint)->priv->add_conditions_to_statement 			(MIDGARD_QUERY_EXECUTOR (self), MIDGARD_QUERY_EXECUTOR (self)->priv->constraint, sql_stm, base_where);
+		if (MIDGARD_QUERY_EXECUTOR (self)->priv->n_constraints == 1)
+			__add_second_dummy_constraint (sss, operation);
+	} else { 
+		/* no constraints, add dummy '1=1 AND 0<1' to satisfy top constraint group */
+		__add_dummy_constraint (sss, operation); 
+		__add_second_dummy_constraint (sss, operation); 
+	}
 
 	/* Add orders , ORDER BY t1.field... */
 	if (!__query_select_add_orders (self)) 
 		goto return_false;
 
 	/* Exclude deleted */
-	__add_exclude_deleted_constraints (sss, operation);
+	if (MGD_DBCLASS_METADATA_CLASS (klass) && !MIDGARD_QUERY_EXECUTOR (self)->priv->include_deleted)
+		__add_exclude_deleted_constraints (sss, operation);
 
 	/* Add limit, LIMIT x */
 	if (MIDGARD_QUERY_EXECUTOR (self)->priv->limit > 0) {
@@ -497,6 +515,7 @@ _midgard_query_select_list_objects (MidgardQuerySelect *self, guint *n_objects)
 {
 	g_return_val_if_fail (self != NULL, NULL);
 	
+	*n_objects = 0;
 	GdaDataModel *model = GDA_DATA_MODEL (MIDGARD_QUERY_EXECUTOR (self)->priv->resultset);
 	if (!model || (model && !GDA_IS_DATA_MODEL (model)))
 		return NULL;
@@ -510,9 +529,9 @@ _midgard_query_select_list_objects (MidgardQuerySelect *self, guint *n_objects)
 	MidgardDBObjectClass *klass = MIDGARD_QUERY_EXECUTOR (self)->priv->storage->priv->klass;
 	MidgardDBObject **objects = g_new (MidgardDBObject *, rows+1);
 
-	for (i = 0; i < rows; i++) {
-		objects[i] = g_object_new (G_OBJECT_CLASS_TYPE (klass), NULL);
-		MIDGARD_DBOBJECT(objects[i])->dbpriv->mgd = mgd;
+	for (i = 0; i < rows; i++) {	
+		objects[i] = g_object_new (G_OBJECT_CLASS_TYPE (klass), "connection", mgd, NULL);
+		MGD_OBJECT_IN_STORAGE (objects[i]) = TRUE;
 		MIDGARD_DBOBJECT(objects[i])->dbpriv->datamodel = model;
 		MIDGARD_DBOBJECT(objects[i])->dbpriv->row = i;
 		g_object_ref (model);
@@ -534,6 +553,33 @@ _midgard_query_select_list_objects (MidgardQuerySelect *self, guint *n_objects)
 	return objects;
 }
 
+static void
+_midgard_query_select_include_deleted (MidgardQuerySelect *self, gboolean toggle)
+{
+	g_return_if_fail (self != NULL);
+	MIDGARD_QUERY_EXECUTOR (self)->priv->include_deleted = toggle;
+}
+
+/**
+ * midgard_query_select_include_deleted:
+ * @self: #MidgardQuerySelect instance
+ * @toggle: toggle to include or exclude deleted objects
+ *
+ * By default, #MidgardQuerySelect ignores deleted objects.
+ * With this method, you can set deleted objects toggle, so such can be 
+ * included in execute results. This method may be called as many times 
+ * as needed, to include (@TRUE) or exclude (@FALSE) deleted objects. 
+ *
+ * Since: 10.05.1
+ */ 
+void
+midgard_query_select_include_deleted (MidgardQuerySelect *self, gboolean toggle)
+{
+	g_return_if_fail (self != NULL);
+	MIDGARD_QUERY_SELECT_GET_CLASS (self)->include_deleted (self, toggle);
+}
+
+
 /**
  * midgard_query_select_list_objects:
  * @self: #MidgardQuerySelect instance
@@ -541,7 +587,7 @@ _midgard_query_select_list_objects (MidgardQuerySelect *self, guint *n_objects)
  *
  * List all objects for which data has been returned during execution.
  *
- * Returns: newly allocated array of #MidgardDBObject
+ * Returns: (element-type MidgardDBObject) (array length=1): newly allocated array of #MidgardDBObject
  * Since: 10.05
  */ 
 MidgardDBObject **
@@ -577,6 +623,11 @@ midgard_query_select_toggle_read_only (MidgardQuerySelect *self, gboolean toggle
 
 /* GOBJECT ROUTINES */
 
+enum {
+	PROPERTY_CONNECTION = 1,
+	PROPERTY_STORAGE
+};
+
 static GObjectClass *parent_class= NULL;
 
 static GObject *
@@ -611,12 +662,62 @@ _midgard_query_select_finalize (GObject *object)
 	parent_class->finalize (object);
 }
 
+static void
+__midgard_query_select_get_property (GObject *object, guint property_id,
+		GValue *value, GParamSpec *pspec)
+{
+	MidgardQuerySelect *self = (MidgardQuerySelect *) object;
+
+	switch (property_id) {
+		
+		case PROPERTY_CONNECTION:
+			/* write and constructor only */
+			break;
+
+		case PROPERTY_STORAGE:
+			/* write and constructor only */
+			break;
+
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (self, property_id, pspec);
+			break;
+	}
+}
+
+static void
+__midgard_query_select_set_property (GObject *object, guint property_id,
+		const GValue *value, GParamSpec *pspec)
+{
+	MidgardQuerySelect *self = (MidgardQuerySelect *) (object);
+	GObject *mgd;
+	GObject *storage;
+
+	switch (property_id) {
+
+		case PROPERTY_CONNECTION:
+			if (!G_VALUE_HOLDS_OBJECT (value))
+				return;
+			MIDGARD_QUERY_EXECUTOR (self)->priv->mgd = g_value_dup_object (value);	
+			break;
+
+		case PROPERTY_STORAGE:
+			if (!G_VALUE_HOLDS_OBJECT (value))
+				return;
+			MIDGARD_QUERY_EXECUTOR (self)->priv->storage = g_value_dup_object (value);
+			break;
+
+  		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (self, property_id, pspec);
+			break;
+	}
+}
+
 static void 
 _midgard_query_select_class_init (MidgardQuerySelectClass *klass, gpointer class_data)
 {
        	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	parent_class = g_type_class_peek_parent (klass);
-	MidgardQueryExecutorClass *executor_class = MIDGARD_QUERY_EXECUTOR_CLASS (klass);
+	MidgardQueryExecutorClass *executor_class = MIDGARD_QUERY_EXECUTOR_CLASS (klass);	
 
 	object_class->constructor = _midgard_query_select_constructor;
 	object_class->dispose = _midgard_query_select_dispose;
@@ -632,6 +733,35 @@ _midgard_query_select_class_init (MidgardQuerySelectClass *klass, gpointer class
 	
 	klass->list_objects = _midgard_query_select_list_objects;
 	klass->toggle_read_only = _midgard_query_select_toggle_read_only;
+	klass->include_deleted = _midgard_query_select_include_deleted;
+
+	object_class->set_property = __midgard_query_select_set_property;
+	object_class->get_property = __midgard_query_select_get_property;
+
+	/* Properties */
+	GParamSpec *pspec = g_param_spec_object ("connection",
+			"", "", 
+			MIDGARD_TYPE_CONNECTION,
+			G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+	/**
+	 * MidgardQuerySelect:connection:
+	 * 
+	 * Pointer for a connection, #MidgardQuerySelect has been initialized for
+	 * 
+	 */  
+	g_object_class_install_property (object_class, PROPERTY_CONNECTION, pspec);
+
+	pspec = g_param_spec_object ("storage",
+			"", "", 
+			MIDGARD_TYPE_QUERY_STORAGE,
+			G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+	/**
+	 * MidgardQuerySelect:storage:
+	 * 
+	 * Pointer for a connection, #MidgardQuerySelect has been initialized for
+	 * 
+	 */  
+	g_object_class_install_property (object_class, PROPERTY_STORAGE, pspec);
 }
 
 GType
